@@ -1,4 +1,5 @@
 import unittest
+import os
 from mw_python_sdk import download_file
 
 
@@ -18,6 +19,18 @@ class TestDownloadFile(unittest.TestCase):
         except Exception as err:
             print(f"An error occurred: {err}")
 
+    def test_download_local(self):
+        dataset_id_tmp = "64e6c644c60d2823b0a2e266"
+        try:  # Example assertion
+            download_file(
+                dataset_id_tmp, "flask.zip", local_dir=os.getcwd()+("/downloads")
+            )
+            download_file(
+                dataset_id_tmp, "flask.zip", local_dir=os.getcwd()+("/downloads")
+            )
+        except Exception as err:
+            print(f"An error occurred: {err}")
+
 
 if __name__ == "__main__":
     unittest.main()
